# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
# Use Python.asdl to test bad ast.

import unittest, support, ast

class TestBad(support.SchemaTestCase):
    def test_module_missing_body(self):
        the_ast = ast.Module()
        with self.capture_error() as cap:
            self.schema.verify(the_ast)
        self.assertEqual(str(cap.exception),
                         "At Module.body: Missing field")

    def test_missing_expr_context(self):
        a_name = ast.Name(id='x') # missing expr_context
        a_binop = ast.BinOp(left=a_name, right =a_name, op=ast.Add())
        the_ast = ast.Expr(value=a_binop)
        with self.capture_error() as cap:
            self.schema.verify(the_ast)
        self.assertEqual(str(cap.exception),
                         "At Name.ctx: Missing field")

    def test_wrong_arg(self):
        bad = ast.Raise() # doesn't matter what is inside
        args = ast.arguments(args=[bad], defaults=[])
        the_ast = ast.FunctionDef(name="haha",
                                  args=args,
                                  body=[],
                                  decorator_list=[])
        with self.capture_error() as cap:
            self.schema.verify(the_ast)
        self.assertEqual(str(cap.exception),
                        "At arguments.args[0]: Cannot be a Raise")

    def test_return_missing_lineno(self):
        the_ast = ast.Return(col_offset=0)
        with self.capture_error() as cap:
            self.schema.verify(the_ast)
        self.assertEqual(str(cap.exception),
                         "At Return.lineno: Missing field")


if __name__ == '__main__':
    unittest.main()




from numba import *
import numba as nb

@autojit
def index(type):
    """
    >>> index(int_)
    ['(0, 1, 2)', '0', '1', '2']
    >>> assert index(int_) == index.py_func(int_)

    >>> index(float_)
    ['(0.0, 1.0, 2.0)', '0.0', '1.0', '2.0']
    >>> assert index(float_) == index.py_func(float_)

    >>> index(complex128)
    ['(0j, (1+0j), (2+0j))', '0j', '(1+0j)', '(2+0j)']
    >>> assert index(complex128) == index.py_func(complex128)
    """
    ttuple = nb.typedtuple(type, (0, 1, 2))
    return map(str, [ttuple, ttuple[0], ttuple[1], ttuple[2]])

@autojit
def index_error(type):
    """
    >>> index_error(int_)
    Traceback (most recent call last):
        ...
    IndexError: list index out of range

    >>> index_error(float_)
    Traceback (most recent call last):
        ...
    IndexError: list index out of range
    """
    ttuple = nb.typedtuple(type, (0, 1, 2))
    return ttuple[4]

@autojit
def from_iterable(type, iterable):
    """
    >>> from_iterable(int_, [1, 2, 3])
    (1, 2, 3)
    >>> from_iterable(int_, (1, 2, 3))
    (1, 2, 3)
    >>> from_iterable(int_, (x for x in [1, 2, 3]))
    (1, 2, 3)

    >>> from_iterable(float_, [1, 2, 3])
    (1.0, 2.0, 3.0)
    >>> from_iterable(float_, (1, 2, 3))
    (1.0, 2.0, 3.0)
    >>> from_iterable(float_, (x for x in [1, 2, 3]))
    (1.0, 2.0, 3.0)

    >>> from_iterable(int_, [1, object(), 3])
    Traceback (most recent call last):
        ...
    TypeError: an integer is required

    >>> from_iterable(int_, object())
    Traceback (most recent call last):
        ...
    TypeError: 'object' object is not iterable
    """
    return nb.typedtuple(type, iterable)

@autojit
def test_count(type):
    """
    >>> test_count(int_)
    (0L, 1L, 2L)
    """
    ttuple = nb.typedtuple(type, [1, 2, 3, 4, 5, 1, 2])
    return ttuple.count(0), ttuple.count(3), ttuple.count(1)

def test(module):
    nb.testmod(module, runit=True)

if __name__ == "__main__":
    import __main__ as module
else:
    import test_typed_tuple as module

test(module)
__test__ = {}


"""
>>> test_staticmethods()
>>> test_classmethods()
>>> obj = Base(10.0)
>>> obj.getvalue(10)
Traceback (most recent call last):
TypeError: function takes exactly 1 argument (2 given)
>>> obj.static1(10)
Traceback (most recent call last):
TypeError: function takes exactly 0 arguments (1 given)
>>> obj.class1(10)
Traceback (most recent call last):
TypeError: function takes exactly 1 argument (2 given)
"""

import sys
from numba import *
from nose.tools import raises

@jit
class Base(object):

    @void(double)
    def __init__(self, myfloat):
        self.value = myfloat

    @double()
    def getvalue(self):
        return self.value
#
    @staticmethod
    @double()
    def static1():
        return 10.0

    @double(double)
    @staticmethod
    def static2(value):
        return value * 2

    @double(double, double)
    @staticmethod
    def static3(value1, value2):
        return value1 * value2

    @staticmethod
    @double(double, double)
    def static4(value1, value2):
        return value1 * value2

    @classmethod
    @double()
    def class1(cls):
        return 10.0

    @double(double)
    @classmethod
    def class2(cls, value):
        return value * 2

    @double(double, double)
    @classmethod
    def class3(cls, value1, value2):
        return value1 * value2

    @classmethod
    @double(double, double)
    def class4(cls, value1, value2):
        return value1 * value2

@jit
class Derived(Base):
    pass

def test_staticmethods():
    def test(obj):
        assert obj.static1() == 10.0
        assert obj.static2(10.0) == 20.0
        assert obj.static3(5.0, 6.0) == 30.0
        assert obj.static4(5.0, 6.0) == 30.0

    test(Base(2.0))
    test(Derived(2.0))
    test(Base)
    test(Derived)

def test_classmethods():
    def test(obj):
        assert obj.class1() == 10.0
        assert obj.class2(10.0) == 20.0
        assert obj.class3(5.0, 6.0) == 30.0
        assert obj.class4(5.0, 6.0) == 30.0

    test(Base(2.0))
    test(Derived(2.0))
    test(Base)
    test(Derived)

if __name__ == '__main__':
#    test_staticmethods()
#    test_classmethods()
    import numba
    numba.testmod()



# -*- coding: utf-8 -*-
"""
Special compiler-recognized numba functions and attributes.
"""
from __future__ import print_function, division, absolute_import

__all__ = ['NULL', 'typeof']

class NumbaDotNULL(object):
    "NULL pointer"

NULL = NumbaDotNULL()

def typeof(variable):
    """
    Get the type of a variable.

    Used outside of Numba code, infers the type for the object.
    """
    from numba.environment import NumbaEnvironment
    context = NumbaEnvironment.get_environment().context
    return context.typemapper.from_python(variable)



# -*- coding: utf-8 -*-
"""
Compiling extension classes works as follows:

    * Create an extension Numba/minivect type holding a symtab
    * Capture attribute types in the symtab ...

        * ... from the class attributes:

            @jit
            class Foo(object):
                attr = double

        * ... from __init__

            @jit
            class Foo(object):
                def __init__(self, attr):
                    self.attr = double(attr)

    * Type infer all methods
    * Compile all extension methods

        * Process signatures such as @void(double)
        * Infer native attributes through type inference on __init__
        * Path the extension type with a native attributes struct
        * Infer types for all other methods
        * Update the ext_type with a vtab type
        * Compile all methods

    * Create descriptors that wrap the native attributes
    * Create an extension type:

      {
        PyObject_HEAD
        ...
        virtual function table (func **)
        native attributes
      }

    The virtual function table (vtab) is a ctypes structure set as
    attribute of the extension types. Objects have a direct pointer
    for efficiency.

See also extension_types.pyx
"""
from __future__ import print_function, division, absolute_import


import types
import ctypes
import logging
import warnings
import inspect

import numba
from numba import pipeline, error, symtab
from numba import typesystem
from numba.minivect import minitypes
from . import extension_types

logger = logging.getLogger(__name__)

#------------------------------------------------------------------------
# Parse method signatures
#------------------------------------------------------------------------

def validate_method(py_func, sig, is_static):
    assert isinstance(py_func, types.FunctionType)
    nargs = py_func.__code__.co_argcount - 1 + is_static
    if len(sig.args) != nargs:
        raise error.NumbaError(
            "Expected %d argument types in function "
            "%s (don't include 'self')" % (nargs, py_func.__name__))

def get_signature(ext_type, is_class, is_static, sig):
    """
    Create a signature given the user-specified signature. E.g.

        class Foo(object):
            @void()                 # becomes: void(ext_type(Foo))
            def method(self): ...
    """
    if is_static:
        leading_arg_types = ()
    elif is_class:
        leading_arg_types = (numba.object_,)
    else:
        leading_arg_types = (ext_type,)

    argtypes = leading_arg_types + sig.args
    restype = sig.return_type
    return minitypes.FunctionType(return_type=restype, args=argtypes)

def get_classmethod_func(func):
    """
    Get the Python function the classmethod or staticmethod is wrapping.

    In Python2.6 classmethod and staticmethod don't have the '__func__'
    attribute.
    """
    if isinstance(func, classmethod):
        return func.__get__(object()).__func__
    else:
        assert isinstance(func, staticmethod)
        return func.__get__(object())

def _process_signature(ext_type, method, default_signature,
                       is_static=False, is_class=False):
    """
    Verify a method signature.

    Returns a Method object and the resolved signature.
    """
    if isinstance(method, types.FunctionType):
        # Process function
        if default_signature is None:
            # TODO: support deferred type inference and autojit methods
            raise error.NumbaError(
                "Method '%s' does not have signature" % (method.__name__,))
        validate_method(method, default_signature or numba.object_(),
                        is_static)
        if default_signature is None:
            default_signature = minitypes.FunctionType(return_type=None,
                                                       args=[])
        sig = get_signature(ext_type, is_class, is_static, default_signature)
        return Method(method, is_class, is_static), sig.return_type, sig.args
    elif isinstance(method, minitypes.Function):
        # @double(...)
        # def func(self, ...): ...
        return _process_signature(ext_type, method.py_func,
                                  method.signature, is_static, is_class)
    else:
        # Process staticmethod and classmethod
        if isinstance(method, staticmethod):
            is_static = True
        elif isinstance(method, classmethod):
            is_class = True
        else:
            return None, None, None

        return _process_signature(
                    ext_type, get_classmethod_func(method),
                    default_signature, is_static=is_static, is_class=is_class)

class Method(object):
    """
    py_func: the python 'def' function
    """

    def __init__(self, py_func, is_class, is_static):
        self.py_func = py_func
        py_func.live_objects = []
        self.is_class = is_class
        self.is_static = is_static
        self.name = py_func.__name__

    def result(self, py_func):
        if self.is_class:
            return classmethod(py_func)
        elif self.is_static:
            return staticmethod(py_func)
        else:
            return py_func

#------------------------------------------------------------------------
# Populate Extension Type with Methods
#------------------------------------------------------------------------

def _process_method_signatures(class_dict, ext_type):
    """
    Process all method signatures:

        * Verify signatures
        * Populate ext_type with method signatures (ExtMethodType)
    """
    for method_name, method in class_dict.iteritems():
        default_signature = None
        if (method_name == '__init__' and
                isinstance(method, types.FunctionType)):
            if inspect.getargspec(method).args:
                warnings.warn(
                    "Constructor for class '%s' has no signature, "
                    "assuming arguments have type 'object'" %
                                        ext_type.py_class.__name__)
            argtypes = [numba.object_] * (method.__code__.co_argcount - 1)
            default_signature = numba.void(*argtypes)

        method, restype, argtypes = _process_signature(ext_type, method,
                                                       default_signature)
        if method is None:
            continue

        signature = typesystem.ExtMethodType(
                    return_type=restype, args=argtypes, name=method.name,
                    is_class=method.is_class, is_static=method.is_static)
        ext_type.add_method(method_name, signature)
        class_dict[method_name] = method

def _type_infer_method(env, ext_type, method, method_name, class_dict, flags):
    if method_name not in ext_type.methoddict:
        return

    signature = ext_type.get_signature(method_name)
    restype, argtypes = signature.return_type, signature.args

    class_dict[method_name] = method
    func_signature, symtab, ast = pipeline.infer_types2(
                        env, method.py_func, restype, argtypes, **flags)
    ext_type.add_method(method_name, func_signature)

def _type_infer_init_method(env, class_dict, ext_type, flags):
    initfunc = class_dict.get('__init__', None)
    if initfunc is None:
        return

    _type_infer_method(env, ext_type, initfunc, '__init__', class_dict, flags)

def _type_infer_methods(env, class_dict, ext_type, flags):
    for method_name, method in class_dict.iteritems():
        if method_name in ('__new__', '__init__') or method is None:
            continue

        _type_infer_method(env, ext_type, method, method_name, class_dict, flags)

def _compile_methods(class_dict, env, ext_type, lmethods, method_pointers,
                     flags):
    parent_method_pointers = getattr(
                    ext_type.py_class, '__numba_method_pointers', None)
    for i, (method_name, func_signature) in enumerate(ext_type.methods):
        if method_name not in class_dict:
            # Inherited method
            assert parent_method_pointers is not None
            name, p = parent_method_pointers[i]
            assert name == method_name
            method_pointers.append((method_name, p))
            continue

        method = class_dict[method_name]
        # Don't use compile_after_type_inference, re-infer, since we may
        # have inferred some return types
        # TODO: delayed types and circular calls/variable assignments
        logger.debug(method.py_func)
        func_env = pipeline.compile2(
            env, method.py_func, func_signature.return_type,
            func_signature.args, name=method.py_func.__name__,
            **flags)
        lmethods.append(func_env.lfunc)
        method_pointers.append((method_name, func_env.translator.lfunc_pointer))
        class_dict[method_name] = method.result(func_env.numba_wrapper_func)

#------------------------------------------------------------------------
# Build Attributes Struct
#------------------------------------------------------------------------

def _construct_native_attribute_struct(ext_type):
    """
    Create attribute struct type from symbol table.
    """
    attrs = dict((name, var.type) for name, var in ext_type.symtab.iteritems())
    if ext_type.attribute_struct is None:
        # No fields to inherit
        ext_type.attribute_struct = numba.struct(**attrs)
    else:
        # Inherit fields from parent
        fields = []
        for name, variable in ext_type.symtab.iteritems():
            if name not in ext_type.attribute_struct.fielddict:
                fields.append((name, variable.type))
                ext_type.attribute_struct.fielddict[name] = variable.type

        # Sort fields by rank
        fields = numba.struct(fields).fields
        ext_type.attribute_struct.fields.extend(fields)

def _create_descr(attr_name):
    """
    Create a descriptor that accesses the attribute on the ctypes struct.
    """
    def _get(self):
        return getattr(self._numba_attrs, attr_name)
    def _set(self, value):
        return setattr(self._numba_attrs, attr_name, value)
    return property(_get, _set)

def inject_descriptors(env, py_class, ext_type, class_dict):
    "Cram descriptors into the class dict"
    for attr_name, attr_type in ext_type.symtab.iteritems():
        descriptor = _create_descr(attr_name)
        class_dict[attr_name] = descriptor

#------------------------------------------------------------------------
# Attribute Inheritance
#------------------------------------------------------------------------

def is_numba_class(cls):
    return hasattr(cls, '__numba_struct_type')

def verify_base_class_compatibility(cls, struct_type, vtab_type):
    "Verify that we can build a compatible class layout"
    bases = [cls]
    for base in cls.__bases__:
        if is_numba_class(base):
            attr_prefix = base.__numba_struct_type.is_prefix(struct_type)
            method_prefix = base.__numba_vtab_type.is_prefix(vtab_type)
            if not attr_prefix or not method_prefix:
                raise error.NumbaError(
                            "Multiple incompatible base classes found: "
                            "%s and %s" % (base, bases[-1]))

            bases.append(base)

def inherit_attributes(ext_type, class_dict):
    "Inherit attributes and methods from superclasses"
    cls = ext_type.py_class
    if not is_numba_class(cls):
        # superclass is not a numba class
        return

    struct_type = cls.__numba_struct_type
    vtab_type = cls.__numba_vtab_type
    verify_base_class_compatibility(cls, struct_type, vtab_type)

    # Inherit attributes
    ext_type.attribute_struct = numba.struct(struct_type.fields)
    for field_name, field_type in ext_type.attribute_struct.fields:
        ext_type.symtab[field_name] = symtab.Variable(field_type,
                                                      promotable_type=False)

    # Inherit methods
    for method_name, method_type in vtab_type.fields:
        func_signature = method_type.base_type
        args = list(func_signature.args)
        if not (func_signature.is_class or func_signature.is_static):
            args[0] = ext_type
        func_signature = func_signature.return_type(*args)
        ext_type.add_method(method_name, func_signature)

    ext_type.parent_attr_struct = struct_type
    ext_type.parent_vtab_type = vtab_type

def process_class_attribute_types(ext_type, class_dict):
    """
    Process class attribute types:

        @jit
        class Foo(object):

            attr = double
    """
    for name, value in class_dict.iteritems():
        if isinstance(value, minitypes.Type):
            ext_type.symtab[name] = symtab.Variable(value, promotable_type=False)

#------------------------------------------------------------------------
# Compile Methods and Build Attributes
#------------------------------------------------------------------------

def compile_extension_methods(env, py_class, ext_type, class_dict, flags):
    """
    Compile extension methods:

        1) Process signatures such as @void(double)
        2) Infer native attributes through type inference on __init__
        3) Path the extension type with a native attributes struct
        4) Infer types for all other methods
        5) Update the ext_type with a vtab type
        6) Compile all methods
    """
    method_pointers = []
    lmethods = []

    class_dict['__numba_py_class'] = py_class

    _process_method_signatures(class_dict, ext_type)
    _type_infer_init_method(env, class_dict, ext_type, flags)
    _construct_native_attribute_struct(ext_type)
    _type_infer_methods(env, class_dict, ext_type, flags)

    # TODO: patch method call types

    # Set vtab type before compiling
    ext_type.vtab_type = numba.struct(
        [(field_name, field_type.pointer())
         for field_name, field_type in ext_type.methods])
    _compile_methods(class_dict, env, ext_type, lmethods, method_pointers,
                     flags)
    return method_pointers, lmethods

#------------------------------------------------------------------------
# Virtual Methods
#------------------------------------------------------------------------

def vtab_name(field_name):
    "Mangle method names for the vtab (ctypes doesn't handle this)"
    if field_name.startswith("__") and field_name.endswith("__"):
        field_name = '__numba_' + field_name.strip("_")
    return field_name

def build_vtab(vtab_type, method_pointers):
    """
    Create ctypes virtual method table.

    vtab_type: the vtab struct type (typesystem.struct)
    method_pointers: a list of method pointers ([int])
    """
    assert len(method_pointers) == len(vtab_type.fields)

    vtab_ctype = numba.struct(
        [(vtab_name(field_name), field_type)
            for field_name, field_type in vtab_type.fields]).to_ctypes()

    methods = []
    for (method_name, method_pointer), (field_name, field_type) in zip(
                                        method_pointers, vtab_type.fields):
        assert method_name == field_name
        method_type_p = field_type.to_ctypes()
        cmethod = ctypes.cast(ctypes.c_void_p(method_pointer), method_type_p)
        methods.append(cmethod)

    vtab = vtab_ctype(*methods)
    return vtab, vtab_type

#------------------------------------------------------------------------
# Build Extension Type
#------------------------------------------------------------------------

def create_extension(env, py_class, flags):
    """
    Compile an extension class given the NumbaEnvironment and the Python
    class that contains the functions that are to be compiled.
    """
    flags.pop('llvm_module', None)

    ext_type = typesystem.ExtensionType(py_class)
    class_dict = dict(vars(py_class))

    inherit_attributes(ext_type, class_dict)
    process_class_attribute_types(ext_type, class_dict)

    method_pointers, lmethods = compile_extension_methods(
            env, py_class, ext_type, class_dict, flags)
    inject_descriptors(env, py_class, ext_type, class_dict)

    vtab, vtab_type = build_vtab(ext_type.vtab_type, method_pointers)

    logger.debug("struct: %s" % ext_type.attribute_struct)
    logger.debug("ctypes struct: %s" % ext_type.attribute_struct.to_ctypes())
    extension_type = extension_types.create_new_extension_type(
            py_class.__name__, py_class.__bases__, class_dict,
            ext_type, vtab, vtab_type,
            lmethods, method_pointers)
    return extension_type



import sys

from numba import extension_types
from numba import *

def format_str(msg, *values):
    return msg % values

@jit
class MyExtension(object):
    """
    >>> obj = MyExtension(10.0)
    >>> obj.value
    10.0
    >>> obj._numba_attrs.value
    10.0
    >>> obj.setvalue(20.0)
    >>> obj.getvalue()
    20.0
    >>> obj.value
    20.0
    >>> obj.getvalue.__name__
    'getvalue'
    >>> obj.getvalue.__doc__
    'Return value'
    >>> type(obj.getvalue.__func__)
    <type 'numba_function_or_method'>
    >>> obj._numba_attrs._fields_
    [('value', <class 'ctypes.c_double'>)]
    """

    @void(double)
    def __init__(self, myfloat):
        self.value = myfloat

    @double()
    def getvalue(self):
        "Return value"
        return self.value

    @void(double)
    def setvalue(self, value):
        "Set value"
        self.value = value

    @object_()
    def __repr__(self):
        return format_str('MyExtension%s', self.value)

@jit
class ObjectAttrExtension(object):
    """
    >>> obj = ObjectAttrExtension(10.0, 'blah')
    Traceback (most recent call last):
        ...
    TypeError: a float is required
    >>> obj = ObjectAttrExtension(10.0, 3.5)
    >>> obj.value1
    10.0
    >>> obj = ObjectAttrExtension('hello', 9.3)
    >>> obj.value1
    'hello'
    >>> obj.setvalue(20.0)
    >>> obj.getvalue()
    20.0

    >>> obj.value1 = MyExtension(10.0)
    >>> obj.value1
    MyExtension10.0
    >>> obj.getvalue()
    MyExtension10.0
    >>> obj.method()
    MyExtension10.0

    >>> obj.method2(15.0)
    30.0

    # This leads to a segfault, why?
#    >>> obj._numba_attrs._fields_
#    [('value2', <class 'ctypes.c_double'>), ('value1', <class 'ctypes.py_object'>)]
    """

    def __init__(self, value1, value2):
        self.value1 = object_(value1)
        self.value2 = double(value2)

    @object_()
    def getvalue(self):
        "Return value"
        return self.value1

    @void(double)
    def setvalue(self, value):
        "Set value"
        self.value1 = value

    @object_()
    def method(self):
        return self.getvalue()

    @object_(int32)
    def method2(self, new_value):
        self.setvalue(new_value * 2)
        result = self.method()
        return result

exttype = ObjectAttrExtension.exttype

@jit
class ExtensionTypeAsAttribute(object):
    """
    >>> print(ExtensionTypeAsAttribute.exttype)
    <Extension ExtensionTypeAsAttribute({'attr': <Extension ObjectAttrExtension>})>
    """

    def __init__(self, attr):
        self.attr = exttype(attr)


if __name__ == '__main__':
#    print ExtensionTypeAsAttribute.exttype
    import numba
    numba.testmod()



"""
>>> Base(10, 11.0).value1
10.0
>>> Base(10, 11.0).value2
11


>>> obj = Derived(10, 11.0)
>>> obj.setvalue(9)
>>> obj.value3
9.0
"""

import sys
import numba
from numba import *
from nose.tools import raises

@jit
class Base(object):

    value1 = double
    value2 = int_

    @void(int_, double)
    def __init__(self, value1, value2):
        self.value1 = value1
        self.value2 = value2

@jit
class Derived(Base):

    value3 = float_

    @void(int_)
    def setvalue(self, value):
        self.value3 = value

if __name__ == '__main__':
    import numba
    numba.testmod()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import logging
import os
import sys
import functools
from numba import environment
from numba import llvm_types
import llvm.core as lc
import llvm.ee as le
logger = logging.getLogger(__name__)

__all__ = ['which', 'find_linker', 'find_args', 'find_shared_ending',
           'Compiler',
           ]

NULL = lc.Constant.null(llvm_types._void_star)
zero = lc.Constant.int(llvm_types._int32, 0)
METH_VARARGS = lc.Constant.int(llvm_types._int32, 1) # == METH_VARARGS
method_def_ty = lc.Type.struct(
    (llvm_types._int8_star, llvm_types._void_star, llvm_types._int32,
     llvm_types._int8_star))
method_def_ptr = lc.Type.pointer(method_def_ty)
py_init_module_fn_ty = lc.Type.function(llvm_types._pyobject_head_struct_p, [
        llvm_types._int8_star, method_def_ptr, llvm_types._int8_star,
        llvm_types._pyobject_head_struct_p, llvm_types._int32])

def which(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, fname)
            if is_exe(exe_file):
                return exe_file
    return None

_configs = {
    'win' : ("link.exe", ("/dll",), '.dll'),
    'dar': ("libtool", ("-dynamic", "-undefined", "dynamic_lookup"), '.so'),
    'default': ("ld", ("-shared",), ".so")
}

def get_configs(arg):
    return _configs.get(sys.platform[:3], _configs['default'])[arg]

find_linker = functools.partial(get_configs, 0)
find_args = functools.partial(get_configs, 1)
find_shared_ending = functools.partial(get_configs, 2)

class Compiler(object):
    def __init__(self, inputs, module_name='numba_exported'):
        self.inputs = inputs
        self.exported_signatures = {}
        self.module_name = module_name
        self.env = environment.NumbaEnvironment.get_environment()

    def _emit_wrapper_init(self, llvm_module, method_defs):
        # Figure out the Python C API module creation function, and
        # get a LLVM function for it.
        py_init_module_fn_name = "Py_InitModule4"
        if llvm_types._trace_refs_:
            py_init_module_fn_name += "TraceRefs"
        if llvm_types._plat_bits == 64:
            py_init_module_fn_name += "_64"
        py_init_module_fn = llvm_module.add_function(py_init_module_fn_ty,
                                                     py_init_module_fn_name)
        py_init_module_fn.linkage = lc.LINKAGE_EXTERNAL
        # Define a constant string for the module name.
        module_name_init = lc.Constant.stringz(self.module_name)
        module_name_const = llvm_module.add_global_variable(
            module_name_init.type, '.module_name')
        module_name_const.initializer = module_name_init
        module_name_const.linkage = lc.LINKAGE_INTERNAL
        # Finish off the method definition array, and construct it as
        # a module constant.
        sentinel = lc.Constant.struct([NULL, NULL, zero, NULL])
        method_defs.append(sentinel)
        method_array_init = lc.Constant.array(method_def_ty, method_defs)
        method_array = llvm_module.add_global_variable(method_array_init.type,
                                                       '.module_methods')
        method_array.initializer = method_array_init
        method_array.linkage = lc.LINKAGE_INTERNAL
        method_array_ptr = lc.Constant.gep(method_array, [zero, zero])
        # Define the module initialization function.
        module_init_fn_name = "init" + self.module_name
        module_init_fn_ty = lc.Type.function(lc.Type.void(), ())
        module_init_fn = llvm_module.add_function(module_init_fn_ty,
                                                  module_init_fn_name)
        entry = module_init_fn.append_basic_block('Entry')
        builder = lc.Builder.new(entry)
        builder.call(py_init_module_fn,
                     (lc.Constant.gep(module_name_const, [zero, zero]),
                      method_array_ptr,
                      NULL,
                      lc.Constant.null(llvm_types._pyobject_head_struct_p),
                      lc.Constant.int(llvm_types._int32, sys.api_version)))
        builder.ret_void()

    def _cull_exports(self):
        '''
        Read all the exported functions/modules in the translator
        environment, and join them into a single LLVM module.

        Resets the export environment.
        '''
        exports_env = self.env.exports
        self.exported_signatures = exports_env.function_signature_map

        # Create new module containing everything
        ret_val = lc.Module.new(self.module_name)

        # Link in all exported functions
        for submod in exports_env.function_module_map.values():
            ret_val.link_in(submod)

        # Link in cbuilder utilities and string constants
        self.env.context.cbuilder_library.link(ret_val)
        self.env.constants_manager.link(ret_val)

        if exports_env.wrap_exports:
            method_defs = []
            wrappers = exports_env.function_wrapper_map.items()
            for name, (submod, lfunc) in wrappers:
                ret_val.link_in(submod)
                method_name_init = lc.Constant.stringz(name)
                method_name = ret_val.add_global_variable(
                    method_name_init.type, '.method_name')
                method_name.initializer = method_name_init
                method_name.linkage = lc.LINKAGE_EXTERNAL
                method_def_const = lc.Constant.struct([
                        lc.Constant.gep(method_name, [zero, zero]),
                        lc.Constant.bitcast(lfunc, llvm_types._void_star),
                        METH_VARARGS, NULL])
                method_defs.append(method_def_const)
            self._emit_wrapper_init(ret_val, method_defs)
        exports_env.reset()
        return ret_val

    def _process_inputs(self, wrap=False, **kws):
        self.env.exports.wrap_exports = wrap
        for ifile in self.inputs:
            exec(compile(open(ifile).read(), ifile, 'exec'))

    def write_llvm_bitcode(self, output, **kws):
        self._process_inputs(**kws)
        lmod = self._cull_exports()
        with open(output, 'wb') as fout:
            lmod.to_bitcode(fout)

    def write_native_object(self, output, **kws):
        self._process_inputs(**kws)
        lmod = self._cull_exports()
	#print(lmod)
	tm = le.TargetMachine.new(reloc=le.RELOC_PIC, features='-avx')
        with open(output, 'wb') as fout:
            objfile = tm.emit_object(lmod)
            fout.write(objfile)

    def emit_header(self, output):
        from numba.minivect import minitypes

        fname, ext = os.path.splitext(output)
        with open(fname + '.h', 'wb') as fout:
            fout.write(minitypes.get_utility())
            fout.write("\n/* Prototypes */\n")
            for signature in self.exported_signatures.values():
                name = signature.name
                restype = signature.return_type.declare()
                args = ", ".join(argtype.declare()
                                 for argtype in signature.args)
                fout.write("extern %s %s(%s);\n" % (restype, name, args))



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.control_flow.cfstats import *
from numba.reporting import *

def compute_uninitialized_phis(flow):
    """
    Compute potentially uninitialized phi variables.
    """
    maybe_uninitialized = set()

    for phi_var, phi_node in flow.blocks[0].phis.iteritems():
        if phi_var.uninitialized:
            maybe_uninitialized.add(phi_var)

    for block in flow.blocks[1:]:
        for phi_var, phi_node in block.phis:
            if any(x in maybe_uninitialized for x in phi_node.incoming):
                maybe_uninitialized.add(phi_var)

    return maybe_uninitialized

def allow_null(node):
    return False

def check_definitions(flow, compiler_directives):
    flow.initialize()
    flow.reaching_definitions()

    # Track down state
    assignments = set()
    # Node to entry map
    references = {}
    assmt_nodes = set()

    for block in flow.blocks:
        i_state = block.i_input
        for stat in block.stats:
            if not isinstance(stat, (NameAssignment, NameReference)):
                continue

            i_assmts = flow.assmts[stat.entry]
            state = flow.map_one(i_state, stat.entry)
            if isinstance(stat, NameAssignment):
                stat.lhs.cf_state.update(state)
                assmt_nodes.add(stat.lhs)
                i_state = i_state & ~i_assmts.mask
                if stat.is_deletion:
                    i_state |= i_assmts.bit
                else:
                    i_state |= stat.bit
                assignments.add(stat)
                # if stat.rhs is not fake_rhs_expr:
                stat.entry.cf_assignments.append(stat)
            elif isinstance(stat, NameReference):
                references[stat.node] = stat.entry
                stat.entry.cf_references.append(stat)
                stat.node.cf_state.update(state)
                if not allow_null(stat.node):
                    i_state &= ~i_assmts.bit
                state.discard(Uninitialized)
                for assmt in state:
                    assmt.refs.add(stat)

    # Check variable usage
    warn_maybe_uninitialized = compiler_directives['warn.maybe_uninitialized']
    warn_unused_result = compiler_directives['warn.unused_result']
    warn_unused = compiler_directives['warn.unused']
    warn_unused_arg = compiler_directives['warn.unused_arg']

    messages = MessageCollection()

    # assignment hints
    for node in assmt_nodes:
        maybe_null = Uninitialized in node.cf_state
        node.cf_maybe_null = maybe_null
        node.cf_is_null = maybe_null and len(node.cf_state) == 1

    # Find uninitialized references and cf-hints
    for node, entry in references.iteritems():
        if Uninitialized in node.cf_state:
            node.cf_maybe_null = True
            from_closure = False # entry.from_closure
            if not from_closure and len(node.cf_state) == 1:
                node.cf_is_null = True
            if allow_null(node) or from_closure: # or entry.is_pyclass_attr:
                pass # Can be uninitialized here
            elif node.cf_is_null:
                is_object = True #entry.type.is_pyobject
                is_unspecified = False #entry.type.is_unspecified
                error_on_uninitialized = False #entry.error_on_uninitialized
                if entry.renameable and (is_object or is_unspecified or
                                         error_on_uninitialized):
                    messages.error(
                        node,
                        "local variable '%s' referenced before assignment"
                        % entry.name)
                else:
                    messages.warning(
                        node,
                        "local variable '%s' referenced before assignment"
                        % entry.name)
            elif warn_maybe_uninitialized:
                messages.warning(
                    node,
                    "local variable '%s' might be referenced before assignment"
                    % entry.name)
        else:
            node.cf_is_null = False
            node.cf_maybe_null = False

    # Unused result
    for assmt in assignments:
        if not assmt.refs: # and not assmt.entry.is_pyclass_attr
        # and not assmt.entry.in_closure):
            if assmt.entry.cf_references and warn_unused_result:
                if assmt.is_arg:
                    messages.warning(assmt, "Unused argument value '%s'" %
                                            assmt.entry.name)
                else:
                    messages.warning(assmt, "Unused result in '%s'" %
                                            assmt.entry.name)
            assmt.lhs.cf_used = False

    # Unused entries
    for entry in flow.entries:
        if (not entry.cf_references and not entry.is_cellvar and
            entry.renameable): # and not entry.is_pyclass_attr
            if entry.is_arg:
                if warn_unused_arg:
                    messages.warning(entry, "Unused argument '%s'" %
                                            entry.name)
            else:
                if warn_unused and entry.warn_unused and flow.is_tracked(entry):
                    messages.warning(entry, "Unused variable '%s'" %
                                            entry.name)
            entry.cf_used = False

    messages.report()

    for node in assmt_nodes:
        node.cf_state = None #ControlFlowState(node.cf_state)
    for node in references:
        node.cf_state = None #ControlFlowState(node.cf_state)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
### Copyright (2012) Continuum Analytics, Inc
### All Rights Reserved

import sys, os, logging
import subprocess, tempfile
import functools

import numba.pycc as pyc

#logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

def get_ending(args):  
    if args.llvm:
        return ".bc"
    if args.olibs:
        return ".o"
    else:
        return pyc.find_shared_ending()

def main(args=[]):
    if not args:
        args = sys.argv
    import argparse
    parser = argparse.ArgumentParser(description="Compile Python modules to a single shared library")
    parser.add_argument("inputs", nargs='+', help="Input file(s)")
    parser.add_argument("-o", nargs=1, dest="output", help="Output file")
    parser.add_argument("-c", action="store_true", dest="olibs",
                        help="Create object file from each input instead of shared-library")
    parser.add_argument("--llvm", action="store_true",
                        help="Emit llvm instead of native code")
    parser.add_argument("--linker", nargs=1, help="Path to linker (default is platform dependent)")
    parser.add_argument("--linker-args", help="Arguments to pass to linker")
    parser.add_argument('--header', action="store_true",
                        help="Emit C header file with function signatures")
    parser.add_argument('--python', action='store_true',
                        help='Emit additionally generated Python wrapper and '
                        'extension module code in output')

    if os.path.basename(args[0]) in ['pycc.py', 'pycc']:
        args = args[1:]

    args = parser.parse_args(args)
    if args.output:
        args.output = args.output[0]
        output_base = os.path.split(args.output)[1]
        module_name = os.path.splitext(output_base)[0]
    else:
        input_base = os.path.splitext(args.inputs[0])[0]
        module_name = os.path.split(input_base)[1]
        args.output = input_base + get_ending(args)
    logger.debug('args.output --> %s', args.output)

    # run the compiler
    logger.debug('inputs --> %s', args.inputs)
    compiler = pyc.Compiler(args.inputs, module_name=module_name)
    if args.llvm:
        logger.debug('emit llvm')
        compiler.write_llvm_bitcode(args.output, wrap=args.python)
    elif args.olibs:
        logger.debug('emit object file')
        compiler.write_native_object(args.output, wrap=args.python)
    else:
        logger.debug('emit shared library')
        logger.debug('write to temporary object file %s', tempfile.gettempdir())
        temp_obj = (tempfile.gettempdir() + os.sep +
                    os.path.basename(args.output) + '.o')
        compiler.write_native_object(temp_obj, wrap=args.python)
        cmdargs = (pyc.find_linker(),) + pyc.find_args() + ('-o', args.output, temp_obj)
        subprocess.check_call(cmdargs)
        os.remove(temp_obj)   # remove temporary object


    if args.header:
        compiler.emit_header(args.output)




from numba import *

@jit(void())
def func1():
    "I am a docstring!"

@autojit
def func2():
    "I am a docstring!"

def module_from_name(func):
    return ".".join(func.__name__.split(".")[:-1])

def name(func):
    return func.__name__.split(".")[-1]

if __name__ == '__main__':
    assert name(func1) == "func1"
    assert func1.__doc__ == "I am a docstring!"
    assert func1.__module__ == module_from_name(func1)

    assert name(func2) == "func2"
    assert func2.__doc__ == "I am a docstring!"
    # This does not yet work for some reason (maybe overridden by PyType_Ready()?)
    # assert func2.__module__ == module_from_name(func2), func2.__module__



from numba import *

#@jit(void(int_)) # directives={'control_flow.dot_output': 'out.dot'})
#@jit(void, [int_], backend='bytecode')
@jit(void(int_), nopython=True)
def func(x):
    i = 0
    #y = 12
    h = 30
    print(i)
    while i < 10:
        if x > i:
            print(x)
            y = 14
        else:
            print(y)

        i = i + 1
        print(y)

    print(i)
    print(y)

#@jit(void())
def _for_loop_fn_0():
    acc = 0.
    for value in range(10):
        acc += value
    return acc

#@jit(void(int_, float_))
def func(a, b):
    if a:
        c = 2
    else:
        c = double(4)

    if a:
        c = 4
    #while a < 4:
    #    for i in range(10):
    #        b = 9
    print(b)

if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.nodes import *

class ExtTypeAttribute(ExprNode):

    _fields = ['value']

    def __init__(self, value, attr, ctx, ext_type, **kwargs):
        super(ExtTypeAttribute, self).__init__(**kwargs)
        self.value = value
        self.attr = attr
        self.variable = ext_type.symtab[attr]
        self.type = self.variable.type
        self.ctx = ctx
        self.ext_type = ext_type

    def __repr__(self):
        return "%s.%s" % (self.value, self.attr)

class NewExtObjectNode(ExprNode):
    """
    Instantiate an extension type. Currently unused.
    """

    _fields = ['args']

    def __init__(self, ext_type, args, **kwargs):
        super(NewExtObjectNode, self).__init__(**kwargs)
        self.ext_type = ext_type
        self.args = args

class ExtensionMethod(ExprNode):

    _fields = ['value']
    call_node = None

    def __init__(self, object, attr, **kwargs):
        super(ExtensionMethod, self).__init__(**kwargs)
        ext_type = object.variable.type
        assert ext_type.is_extension
        self.value = object
        self.attr = attr

        method_type, self.vtab_index = ext_type.methoddict[attr]
        self.type = minitypes.FunctionType(return_type=method_type.return_type,
                                           args=method_type.args,
                                           is_bound_method=True)

    def __repr__(self):
        return "%s.%s" % (self.value, self.attr)


#class ExtensionMethodCall(Node):
#    """
#    Low level call that has resolved the virtual method.
#    """
#
#    _fields = ['vmethod', 'args']
#
#    def __init__(self, vmethod, self_obj, args, signature, **kwargs):
#        super(ExtensionMethodCall, self).__init__(**kwargs)
#        self.vmethod = vmethod
#        self.args = args
#        self.signature = signature
#        self.type = signature
